#ifndef _EDISPZ_CONTROLLER_H_
#define _EDISPZ_CONTROLLER_H_

#include <glib.h>
#include <glib/gstdio.h>
#include "bp35a1-client.h"
#include "bp35a1-message.h"

#define EDISPZ_HEADER_SIZE (15)

typedef struct _EdispzController EdispzController;
typedef struct _EdispzPacket EdispzPacket;

typedef enum
{
  EDISPZ_PACKET_UNKNOWN,
  EDISPZ_PACKET_ACK_NACK,
  EDISPZ_PACKET_NORMAL,
  EDISPZ_PACKET_TEST,
  EDISPZ_PACKET_SPECIAL,
  EDISPZ_PACKET_TIMEKEEPING
} EdispzPacketType;

typedef enum
{
  EDISPZ_PACKET_DATA_UNKNOWN,
  EDISPZ_PACKET_DATA_ACK_NACK,
  EDISPZ_PACKET_DATA_BATTERY,
  EDISPZ_PACKET_DATA_DOOR_IS_OPEN,
  EDISPZ_PACKET_DATA_DOOR_OPEN_COUNT,
  EDISPZ_PACKET_DATA_DOOR_TIME,
  EDISPZ_PACKET_DATA_TEMPERATURE,
  EDISPZ_PACKET_DATA_HUMIDITY,
  EDISPZ_PACKET_DATA_HUMAN_IS_REACT,
  EDISPZ_PACKET_DATA_HUMAN_REACT_COUNT,
  EDISPZ_PACKET_DATA_FIRMWARE_VERSION,
  EDISPZ_PACKET_DATA_TIMEKEEPING_FLAG,
  EDISPZ_PACKET_DATA_FIRMWARE_SEQUENCE_NUMBER,
  EDISPZ_PACKET_DATA_CO2,
  EDISPZ_PACKET_DATA_ANALOG_INPUT,
  EDISPZ_PACKET_DATA_SCALE,
  EDISPZ_PACKET_DATA_PULSE
} EdispzPacketDataType;

EdispzController *edispz_controller_new   (GAsyncQueue *dest_queue);
gint              edispz_controller_start (EdispzController *controller);
gint              edispz_controller_stop  (EdispzController *controller);
void              edispz_controller_free  (EdispzController *controller);
EdispzPacketType  edispz_packet_get_type  (GByteArray *packet);
gchar            *edispz_packet_get_data  (GByteArray *packet, EdispzPacketDataType data_type);
void              delete_edispz_packet    (EdispzPacket *packet);

struct _EdispzPacket
{
  gchar* mac;
  guint type;
  GByteArray *data;
};

struct _EdispzController
{
  gchar *device;
  GAsyncQueue *dest_queue;

  Bp35a1Client *bp35a1_client;

  guchar tid_up, tid_low;
  guint interval_time;
  guint offset_time;
  guint human_sensor_wait_time;
  gushort heartbeat_time;
  guchar door_detect_mode;
  guchar pana_psk[16];
  guchar operation_mode;

  GMutex lock;
  GThread *loop;
  gboolean is_running;
};

#endif
